#version 330

uniform mat4 modelMatrix;
uniform vec3 cameraPosition;

uniform sampler2D colorTexture;
uniform sampler2D shadowMap;

uniform Material {
	vec4 m_diffuse;
	vec4 ambient;
	vec4 specular;
	vec4 emissive;
	float shininess;
	float test;
} material;

#define MAX_LIGHTS 10
uniform int numLights;
uniform struct Light {
   vec4 position;
   vec3 color;
   float attenuation;
   float ambientCE;
   float spotlightAngle;
   vec3 spotlightDirection;
} lights[MAX_LIGHTS];

in vec3 fragVert;
in vec3 fragNormal;
in vec2 fragTexCoord;
in vec4 shadowMappingCoord;

out vec4 finalColor;

vec3 ApplyLight(Light light, vec3 surfaceColor, vec3 normal, vec3 surfacePos, vec3 surfaceToCamera) {
    vec3 surfaceToLight;
    float attenuation = 1.0;
    if(light.position.w == 0.0) {
        //directional light
        surfaceToLight = normalize(light.position.xyz);
        attenuation = 1.0; //no attenuation for directional lights
    } else {
        //point light
        surfaceToLight = normalize(light.position.xyz - surfacePos);
        float distanceToLight = length(light.position.xyz - surfacePos);
        attenuation = 1.0 / (1.0 + light.attenuation * pow(distanceToLight, 2));

        //cone restrictions (affects attenuation)
        float lightToSurfaceAngle = degrees(acos(dot(-surfaceToLight, normalize(light.spotlightDirection))));
        if(lightToSurfaceAngle > light.spotlightAngle){
            attenuation = 0.0;
        }
    }

    //ambient
    vec3 ambient = light.ambientCE * surfaceColor.rgb * light.color;

    //diffuse
    float diffuseCoefficient = max(0.0, dot(normal, surfaceToLight));
    vec3 diffuse = diffuseCoefficient * surfaceColor.rgb * light.color;
    
    //specular
    float specularCoefficient = 0.0;
    if(diffuseCoefficient > 0.0)
        specularCoefficient = pow(max(0.0, dot(surfaceToCamera, reflect(-surfaceToLight, normal))), 80.0);
    vec3 specular = specularCoefficient * vec3(1,1,1) * light.color;
	
    float visibility = 0;
    
    vec3 ProjCoords = shadowMappingCoord.xyz / shadowMappingCoord.w;                                  
	vec2 UVCoords;                                                                          
	UVCoords.x = 0.5 * ProjCoords.x + 0.5;                                                  
	UVCoords.y = 0.5 * ProjCoords.y + 0.5;                                                  
	float z = 0.5 * ProjCoords.z + 0.5;                                                     
	float Depth = texture(shadowMap, UVCoords).x;                                          
		
	// here is the weird part -->
	// BAD: if (Depth < z + 0.0001)
	// was only 3 days to find...
	if (Depth < z - 0.00001)                                                                 
		visibility = 0.5;                                                                         
	else                                                                                    
		visibility = 1.0; 
		
		
	/*vec3 ProjCoords;
    ProjCoords.xy = shadowMappingCoord.xy;
    ProjCoords.z = shadowMappingCoord.z / shadowMappingCoord.w;
    vec2 UVCoords;                                                                          
    UVCoords.x = 0.5 * ProjCoords.x + 0.5;                                                  
    UVCoords.y = 0.5 * ProjCoords.y + 0.5;                                                  
    float z = 0.5 * ProjCoords.z + 0.5;                                                   
        
    float Depth = texture(shadowMap, shadowMappingCoord.xy).x;                                          
    if (Depth < ProjCoords.z + 0.00001)                                                                 
        visibility = 0;                                                                         
    else                                                                                    
    visibility = 1.0;*/
        
        
    //float visibility = texture( shadowMap, vec2(shadowMappingCoord.xy, (shadowMappingCoord.z)/shadowMappingCoord.w) );
    
    //linear color (color before gamma correction)
    return ambient + visibility * attenuation * (diffuse + specular);
}

void main() {
    /*vec3 normal = normalize(transpose(inverse(mat3(modelMatrix))) * fragNormal);
    vec3 surfacePos = vec3(modelMatrix * vec4(fragVert, 1));
    vec4 surfaceColor = texture(colorTexture, fragTexCoord);
    vec3 surfaceToCamera = normalize(cameraPosition - surfacePos);

    //combine color from all the lights
    vec3 linearColor = vec3(0);
    for(int i = 0; i < numLights; ++i){
        linearColor += ApplyLight(lights[i], surfaceColor.rgb, normal, surfacePos, surfaceToCamera);
    }
    
    //final color (after gamma correction)
    vec3 gamma = vec3(1.0/2.2);*/
	vec4 surfaceColor = texture(colorTexture, fragTexCoord);
	float alpha = 1 - surfaceColor.r;
   //finalColor = vec4(pow(linearColor, gamma), alpha);
   finalColor = vec4(surfaceColor.rgb, alpha);
   //finalColor = vec4(1,0,0,1);
}
