#version 330

layout(binding=0) uniform sampler2D uInputTex;
layout(binding=1) uniform sampler2D uDepthTex;

layout(binding=3) uniform sampler2D uVelocityTex;

uniform float uMotionScale = 1.0; // current fps / target fps
uniform int uMaxMotionBlurSamples = 128;

layout(location=0) out vec4 finalColor;






void main() {

	
	
	vec2 velocity = decodeVelocity(texture(uVelocityTex, vTexcoord).rg);
	velocity *= uMotionScale;
	
	//	compute number of blur samples to take:
	vec2 texelSize = 1.0 / vec2(textureSize(uInputTex, 0));
	float speed = length(velocity / texelSize);
	int nSamples = clamp(int(speed), 1, uMaxMotionBlurSamples);
	
	vec4 finalColor = vec4(0.0);
	
	
	velocity = normalize(velocity) * texelSize;
	
	float hlim = float(-nSamples) * 0.5 + 0.5;
	
	for (int i = 0; i < nSamples; ++i) {
			vec2 offset = velocity * (hlim + float(i));
			finalColor += texture(uInputTex, vTexcoord + offset);
	}
	
	
	finalColor /= float(nSamples);
	

}